@@ -59,6 +59,7 @@ struct _XfdashboardActorPrivate
 	/* Instance related */
 	GHashTable		*lastThemeStyleSet;
 	gboolean		forceStyleRevalidation;
+	gboolean		isFirstParent;
 };
 
 /* Properties */
@@ -203,16 +204,16 @@ static void _xfdashboard_actor_on_reactive_changed(GObject *inObject,
 
 	self=XFDASHBOARD_ACTOR(inObject);
 
-	/* Set pseudo-class ':insensitive' if actor is now not reactive
+	/* Add pseudo-class ':insensitive' if actor is now not reactive
 	 * and remove this pseudo-class if actor is now reactive.
 	 */
 	if(clutter_actor_get_reactive(CLUTTER_ACTOR(self)))
 	{
-		xfdashboard_stylable_add_pseudo_class(XFDASHBOARD_STYLABLE(self), "insensitive");
+		xfdashboard_stylable_remove_pseudo_class(XFDASHBOARD_STYLABLE(self), "insensitive");
 	}
 		else
 		{
-			xfdashboard_stylable_remove_pseudo_class(XFDASHBOARD_STYLABLE(self), "insensitive");
+			xfdashboard_stylable_add_pseudo_class(XFDASHBOARD_STYLABLE(self), "insensitive");
 		}
 
 	/* Invalide styling to get it recomputed */
@@ -746,12 +747,14 @@ static gboolean _xfdashboard_actor_enter_event(ClutterActor *inActor, ClutterCro
 /* Actor was (re)parented */
 static void _xfdashboard_actor_parent_set(ClutterActor *inActor, ClutterActor *inOldParent)
 {
-	XfdashboardActor		*self;
-	ClutterActorClass		*parentClass;
+	XfdashboardActor			*self;
+	XfdashboardActorPrivate		*priv;
+	ClutterActorClass			*parentClass;
 
 	g_return_if_fail(XFDASHBOARD_IS_ACTOR(inActor));
 
 	self=XFDASHBOARD_ACTOR(inActor);
+	priv=self->priv;
 
 	/* Call parent's virtual function */
 	parentClass=CLUTTER_ACTOR_CLASS(xfdashboard_actor_parent_class);
@@ -760,6 +763,26 @@ static void _xfdashboard_actor_parent_set(ClutterActor *inActor, ClutterActor *i
 		parentClass->parent_set(inActor, inOldParent);
 	}
 
+	/* Check if it is a newly created actor which is parented for the first time.
+	 * Then emit 'actor-created' signal on stage.
+	 */
+	if(priv->isFirstParent &&
+		!inOldParent &&
+		clutter_actor_get_parent(inActor))
+	{
+		ClutterActor			*stage;
+
+		/* Get stage where this actor belongs to and emit signal at stage */
+		stage=clutter_actor_get_stage(inActor);
+		if(XFDASHBOARD_IS_STAGE(stage))
+		{
+			g_signal_emit_by_name(stage, "actor-created", inActor, NULL);
+		}
+
+		/* Set flag that a parent set and signal was emitted */
+		priv->isFirstParent=FALSE;
+	}
+
 	/* Invalide styling to get it recomputed because its ID (from point
 	 * of view of css) has changed. Also invalidate children as they might
 	 * reference the old, invalid parent or the new, valid one.
@@ -975,6 +998,10 @@ void xfdashboard_actor_class_init(XfdashboardActorClass *klass)
 	xfdashboard_actor_install_stylable_property_by_name(klass, "y-expand");
 	xfdashboard_actor_install_stylable_property_by_name(klass, "x-align");
 	xfdashboard_actor_install_stylable_property_by_name(klass, "y-align");
+	xfdashboard_actor_install_stylable_property_by_name(klass, "margin-top");
+	xfdashboard_actor_install_stylable_property_by_name(klass, "margin-bottom");
+	xfdashboard_actor_install_stylable_property_by_name(klass, "margin-left");
+	xfdashboard_actor_install_stylable_property_by_name(klass, "margin-right");
 }
 
 /* Object initialization
@@ -992,6 +1019,7 @@ void xfdashboard_actor_init(XfdashboardActor *self)
 	priv->styleClasses=NULL;
 	priv->stylePseudoClasses=NULL;
 	priv->lastThemeStyleSet=NULL;
+	priv->isFirstParent=TRUE;
 
 	/* Connect signals */
 	g_signal_connect(self, "notify::mapped", G_CALLBACK(_xfdashboard_actor_on_mapped_changed), NULL);
