@@ -762,12 +762,14 @@ static gboolean _xfdashboard_applications_view_focusable_set_selection(Xfdashboa
 
 	/* Set new selection */
 	priv->selectedItem=inSelection;
+	if(priv->selectedItem)
+	{
+		/* Add weak reference at new selection */
+		g_object_add_weak_pointer(G_OBJECT(priv->selectedItem), &priv->selectedItem);
 
-	/* Add weak reference at new selection */
-	g_object_add_weak_pointer(G_OBJECT(priv->selectedItem), &priv->selectedItem);
-
-	/* Ensure new selection is visible */
-	if(inSelection) xfdashboard_view_child_ensure_visible(XFDASHBOARD_VIEW(self), inSelection);
+		/* Ensure new selection is visible */
+		xfdashboard_view_child_ensure_visible(XFDASHBOARD_VIEW(self), priv->selectedItem);
+	}
 
 	/* New selection was set successfully */
 	return(TRUE);
@@ -1041,6 +1043,7 @@ static ClutterActor* _xfdashboard_applications_view_focusable_find_selection(Xfd
 	XfdashboardApplicationsViewPrivate		*priv;
 	ClutterActor							*selection;
 	ClutterActor							*newSelection;
+	gchar									*valueName;
 
 	g_return_val_if_fail(XFDASHBOARD_IS_FOCUSABLE(inFocusable), NULL);
 	g_return_val_if_fail(XFDASHBOARD_IS_APPLICATIONS_VIEW(inFocusable), NULL);
@@ -1056,10 +1059,13 @@ static ClutterActor* _xfdashboard_applications_view_focusable_find_selection(Xfd
 	if(!inSelection)
 	{
 		newSelection=clutter_actor_get_first_child(CLUTTER_ACTOR(self));
-		g_debug("No selection at %s, so select first child %s for direction %u",
+
+		valueName=xfdashboard_get_enum_value_name(XFDASHBOARD_TYPE_SELECTION_TARGET, inDirection);
+		g_debug("No selection at %s, so select first child %s for direction %s",
 				G_OBJECT_TYPE_NAME(self),
 				newSelection ? G_OBJECT_TYPE_NAME(newSelection) : "<nil>",
-				inDirection);
+				valueName);
+		g_free(valueName);
 
 		return(newSelection);
 	}
@@ -1114,8 +1120,6 @@ static ClutterActor* _xfdashboard_applications_view_focusable_find_selection(Xfd
 
 		default:
 			{
-				gchar					*valueName;
-
 				valueName=xfdashboard_get_enum_value_name(XFDASHBOARD_TYPE_SELECTION_TARGET, inDirection);
 				g_critical(_("Focusable object %s does not handle selection direction of type %s."),
 							G_OBJECT_TYPE_NAME(self),
@@ -1193,9 +1197,21 @@ static void _xfdashboard_applications_view_dispose(GObject *inObject)
 	XfdashboardApplicationsViewPrivate	*priv=self->priv;
 
 	/* Release allocated resources */
-	if(priv->xfconfChannel) priv->xfconfChannel=NULL;
+	if(priv->selectedItem)
+	{
+		g_object_remove_weak_pointer(G_OBJECT(priv->selectedItem), &priv->selectedItem);
+		priv->selectedItem=NULL;
+	}
 
-	if(priv->layout) priv->layout=NULL;
+	if(priv->xfconfChannel)
+	{
+		priv->xfconfChannel=NULL;
+	}
+
+	if(priv->layout)
+	{
+		priv->layout=NULL;
+	}
 
 	if(priv->xfconfShowAllAppsMenuBindingID)
 	{
