@@ -56,6 +56,8 @@ GType xfdashboard_image_content_loading_state_get_type(void) G_GNUC_CONST;
 #define XFDASHBOARD_TYPE_IMAGE_CONTENT_LOADING_STATE	(xfdashboard_image_content_loading_state_get_type())
 
 /* enumerations from "plugin.h" */
+GType xfdashboard_plugin_flag_get_type(void) G_GNUC_CONST;
+#define XFDASHBOARD_TYPE_PLUGIN_FLAG	(xfdashboard_plugin_flag_get_type())
 GType xfdashboard_plugin_error_enum_get_type(void) G_GNUC_CONST;
 #define XFDASHBOARD_TYPE_PLUGIN_ERROR_ENUM	(xfdashboard_plugin_error_enum_get_type())
 
