@@ -30,6 +30,7 @@
 
 #include <glib/gi18n-lib.h>
 
+#include <libxfdashboard/enums.h>
 #include <libxfdashboard/marshal.h>
 #include <libxfdashboard/compat.h>
 
@@ -56,6 +57,7 @@ struct _XfdashboardPluginPrivate
 {
 	/* Properties related */
 	gchar						*id;
+	XfdashboardPluginFlag		flags;
 	gchar						*name;
 	gchar						*description;
 	gchar						*author;
@@ -83,6 +85,7 @@ enum
 	PROP_FILENAME,
 
 	PROP_ID,
+	PROP_FLAGS,
 	PROP_NAME,
 	PROP_DESCRIPTION,
 	PROP_AUTHOR,
@@ -252,6 +255,28 @@ static void _xfdashboard_plugin_set_id(XfdashboardPlugin *self, const gchar *inI
 	}
 }
 
+/* Set flags for plugin */
+static void _xfdashboard_plugin_set_flags(XfdashboardPlugin *self, XfdashboardPluginFlag inFlags)
+{
+	XfdashboardPluginPrivate		*priv;
+
+	g_return_if_fail(XFDASHBOARD_IS_PLUGIN(self));
+	g_return_if_fail(self->priv->flags==XFDASHBOARD_PLUGIN_FLAG_NONE);
+	g_return_if_fail(self->priv->state==XFDASHBOARD_PLUGIN_STATE_NONE);
+
+	priv=self->priv;
+
+	/* Set value if changed */
+	if(priv->flags!=inFlags)
+	{
+		/* Set value */
+		priv->flags=inFlags;
+
+		/* Notify about property change */
+		g_object_notify_by_pspec(G_OBJECT(self), XfdashboardPluginProperties[PROP_FLAGS]);
+	}
+}
+
 /* Set name for plugin */
 static void _xfdashboard_plugin_set_name(XfdashboardPlugin *self, const gchar *inName)
 {
@@ -668,6 +693,10 @@ static void _xfdashboard_plugin_set_property(GObject *inObject,
 			_xfdashboard_plugin_set_id(self, g_value_get_string(inValue));
 			break;
 
+		case PROP_FLAGS:
+			_xfdashboard_plugin_set_flags(self, g_value_get_flags(inValue));
+			break;
+
 		case PROP_NAME:
 			_xfdashboard_plugin_set_name(self, g_value_get_string(inValue));
 			break;
@@ -712,6 +741,10 @@ static void _xfdashboard_plugin_get_property(GObject *inObject,
 			g_value_set_string(outValue, priv->id);
 			break;
 
+		case PROP_FLAGS:
+			g_value_set_flags(outValue, priv->flags);
+			break;
+
 		case PROP_NAME:
 			g_value_set_string(outValue, priv->name);
 			break;
@@ -785,6 +818,14 @@ static void xfdashboard_plugin_class_init(XfdashboardPluginClass *klass)
 							NULL,
 							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY);
 
+	XfdashboardPluginProperties[PROP_FLAGS]=
+		g_param_spec_flags("flags",
+							_("Flags"),
+							_("Flags defining behaviour of this plugin"),
+							XFDASHBOARD_TYPE_PLUGIN_FLAG,
+							XFDASHBOARD_PLUGIN_FLAG_NONE,
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
 	XfdashboardPluginProperties[PROP_NAME]=
 		g_param_spec_string("name",
 							_("name"),
@@ -895,6 +936,7 @@ static void xfdashboard_plugin_init(XfdashboardPlugin *self)
 	priv->lastLoadingError=NULL;
 
 	priv->id=NULL;
+	priv->flags=XFDASHBOARD_PLUGIN_FLAG_NONE;
 	priv->name=NULL;
 	priv->description=NULL;
 	priv->author=NULL;
@@ -1012,6 +1054,14 @@ const gchar* xfdashboard_plugin_get_id(XfdashboardPlugin *self)
 	return(self->priv->id);
 }
 
+/* Get flags of plugin */
+XfdashboardPluginFlag xfdashboard_plugin_get_flags(XfdashboardPlugin *self)
+{
+	g_return_val_if_fail(XFDASHBOARD_IS_PLUGIN(self), XFDASHBOARD_PLUGIN_FLAG_NONE);
+
+	return(self->priv->flags);
+}
+
 /* Set plugin information */
 void xfdashboard_plugin_set_info(XfdashboardPlugin *self,
 									const gchar *inFirstPropertyName, ...)
