@@ -392,8 +392,12 @@ static void _xfdashboard_search_result_container_on_result_item_actor_clicked(Xf
 
 	self=XFDASHBOARD_SEARCH_RESULT_CONTAINER(inUserData);
 
-	/* Activate result item by actor clicked */
-	_xfdashboard_search_result_container_activate_result_item_by_actor(self, inActor);
+	/* Only emit signal if click was perform with left button */
+	if(xfdashboard_click_action_get_button(inAction)==XFDASHBOARD_CLICK_ACTION_LEFT_BUTTON)
+	{
+		/* Activate result item by actor clicked */
+		_xfdashboard_search_result_container_activate_result_item_by_actor(self, inActor);
+	}
 }
 
 /* Get and set up actor for result item from search provider */
@@ -1079,6 +1083,12 @@ static void _xfdashboard_search_result_container_dispose(GObject *inObject)
 	/* Release allocated variables */
 	_xfdashboard_search_result_container_update_selection(self, NULL);
 
+	if(priv->selectedItem)
+	{
+		g_object_remove_weak_pointer(G_OBJECT(priv->selectedItem), &priv->selectedItem);
+		priv->selectedItem=NULL;
+	}
+
 	if(priv->provider)
 	{
 		g_object_unref(priv->provider);
