@@ -80,6 +80,7 @@ struct _XfdashboardWindowsViewPrivate
 	gboolean							isWindowsNumberShown;
 
 	gboolean							filterMonitorWindows;
+	gboolean							filterWorkspaceWindows;
 	XfdashboardStageInterface			*currentStage;
 	XfdashboardWindowTrackerMonitor		*currentMonitor;
 	guint								currentStageMonitorBindingID;
@@ -95,6 +96,7 @@ enum
 	PROP_PREVENT_UPSCALING,
 	PROP_SCROLL_EVENT_CHANGES_WORKSPACE,
 	PROP_FILTER_MONITOR_WINDOWS,
+	PROP_FILTER_WORKSPACE_WINDOWS,
 
 	PROP_LAST
 };
@@ -228,8 +230,13 @@ static gboolean _xfdashboard_windows_view_is_visible_window(XfdashboardWindowsVi
 		return(FALSE);
 	}
 
-	if(!priv->workspace ||
-		!xfdashboard_window_tracker_window_is_visible_on_workspace(inWindow, priv->workspace))
+	if(!priv->workspace)
+	{
+		return(FALSE);
+	}
+
+	if(!xfdashboard_window_tracker_window_is_visible(inWindow) ||
+		(priv->filterWorkspaceWindows && !xfdashboard_window_tracker_window_is_on_workspace(inWindow, priv->workspace)))
 	{
 		return(FALSE);
 	}
@@ -322,8 +329,8 @@ static void _xfdashboard_windows_view_recreate_window_actors(XfdashboardWindowsV
 	if(priv->selectedItem)
 	{
 		g_object_remove_weak_pointer(G_OBJECT(priv->selectedItem), &priv->selectedItem);
+		priv->selectedItem=NULL;
 	}
-	priv->selectedItem=NULL;
 
 	/* Destroy all actors */
 	clutter_actor_destroy_all_children(CLUTTER_ACTOR(self));
@@ -711,16 +718,30 @@ static void _xfdashboard_windows_view_on_window_monitor_changed(XfdashboardWindo
 static void _xfdashboard_windows_view_on_window_clicked(XfdashboardWindowsView *self,
 														gpointer inUserData)
 {
+	XfdashboardWindowsViewPrivate		*priv;
 	XfdashboardLiveWindow				*liveWindow;
 	XfdashboardWindowTrackerWindow		*window;
+	XfdashboardWindowTrackerWorkspace	*activeWorkspace;
+	XfdashboardWindowTrackerWorkspace	*windowWorkspace;
 
 	g_return_if_fail(XFDASHBOARD_IS_WINDOWS_VIEW(self));
 	g_return_if_fail(XFDASHBOARD_IS_LIVE_WINDOW(inUserData));
 
+	priv=self->priv;
 	liveWindow=XFDASHBOARD_LIVE_WINDOW(inUserData);
 
-	/* Activate clicked window */
-	window=XFDASHBOARD_WINDOW_TRACKER_WINDOW(xfdashboard_live_window_get_window(liveWindow));
+	/* Get window to activate */
+	window=xfdashboard_live_window_get_window(liveWindow);
+
+	/* Move to workspace if window to active is on a different one than the active one */
+	activeWorkspace=xfdashboard_window_tracker_get_active_workspace(priv->windowTracker);
+	if(!xfdashboard_window_tracker_window_is_on_workspace(window, activeWorkspace))
+	{
+		windowWorkspace=xfdashboard_window_tracker_window_get_workspace(window);
+		xfdashboard_window_tracker_workspace_activate(windowWorkspace);
+	}
+
+	/* Activate window */
 	xfdashboard_window_tracker_window_activate(window);
 
 	/* Quit application */
@@ -1071,8 +1092,8 @@ static void _xfdashboard_windows_view_set_scroll_event_changes_workspace(Xfdashb
 	}
 }
 
-/* Set flag if this view should show all windows of current workspace or only the windows
- * which are at current workspace and on the monitor where this view is placed at.
+/* Set flag if this view should show all windows of all monitors or only the windows
+ * which are at the monitor where this view is placed at.
  */
 static void _xfdashboard_windows_view_set_filter_monitor_windows(XfdashboardWindowsView *self, gboolean inFilterMonitorWindows)
 {
@@ -1096,6 +1117,31 @@ static void _xfdashboard_windows_view_set_filter_monitor_windows(XfdashboardWind
 	}
 }
 
+/* Set flag if this view should show all windows of all workspaces or only the windows
+ * which are at current workspace.
+ */
+static void _xfdashboard_windows_view_set_filter_workspace_windows(XfdashboardWindowsView *self, gboolean inFilterWorkspaceWindows)
+{
+	XfdashboardWindowsViewPrivate		*priv;
+
+	g_return_if_fail(XFDASHBOARD_IS_WINDOWS_VIEW(self));
+
+	priv=self->priv;
+
+	/* Set value if changed */
+	if(priv->filterWorkspaceWindows!=inFilterWorkspaceWindows)
+	{
+		/* Set value */
+		priv->filterWorkspaceWindows=inFilterWorkspaceWindows;
+
+		/* Recreate all window actors */
+		_xfdashboard_windows_view_recreate_window_actors(self);
+
+		/* Notify about property change */
+		g_object_notify_by_pspec(G_OBJECT(self), XfdashboardWindowsViewProperties[PROP_FILTER_WORKSPACE_WINDOWS]);
+	}
+}
+
 /* Action signal to close currently selected window was emitted */
 static gboolean _xfdashboard_windows_view_window_close(XfdashboardWindowsView *self,
 														XfdashboardFocusable *inSource,
@@ -1419,7 +1465,7 @@ static gboolean _xfdashboard_windows_view_focusable_set_selection(XfdashboardFoc
 	priv->selectedItem=inSelection;
 
 	/* Add weak reference at new selection */
-	g_object_add_weak_pointer(G_OBJECT(priv->selectedItem), &priv->selectedItem);
+	if(priv->selectedItem) g_object_add_weak_pointer(G_OBJECT(priv->selectedItem), &priv->selectedItem);
 
 	/* New selection was set successfully */
 	return(TRUE);
@@ -1443,6 +1489,7 @@ static ClutterActor* _xfdashboard_windows_view_focusable_find_selection(Xfdashbo
 	gint									newSelectionIndex;
 	ClutterActorIter						iter;
 	ClutterActor							*child;
+	gchar									*valueName;
 
 	g_return_val_if_fail(XFDASHBOARD_IS_FOCUSABLE(inFocusable), NULL);
 	g_return_val_if_fail(XFDASHBOARD_IS_WINDOWS_VIEW(inFocusable), NULL);
@@ -1458,10 +1505,13 @@ static ClutterActor* _xfdashboard_windows_view_focusable_find_selection(Xfdashbo
 	if(!inSelection)
 	{
 		newSelection=clutter_actor_get_first_child(CLUTTER_ACTOR(self));
-		g_debug("No selection at %s, so select first child %s for direction %u",
+
+		valueName=xfdashboard_get_enum_value_name(XFDASHBOARD_TYPE_SELECTION_TARGET, inDirection);
+		g_debug("No selection at %s, so select first child %s for direction %s",
 				G_OBJECT_TYPE_NAME(self),
 				newSelection ? G_OBJECT_TYPE_NAME(newSelection) : "<nil>",
-				inDirection);
+				valueName);
+		g_free(valueName);
 
 		return(newSelection);
 	}
@@ -1556,7 +1606,7 @@ static ClutterActor* _xfdashboard_windows_view_focusable_find_selection(Xfdashbo
 
 		case XFDASHBOARD_SELECTION_TARGET_NEXT:
 			newSelection=clutter_actor_get_next_sibling(inSelection);
-			if(!selection) selection=clutter_actor_get_previous_sibling(inSelection);
+			if(!newSelection) newSelection=clutter_actor_get_previous_sibling(inSelection);
 			break;
 
 		case XFDASHBOARD_SELECTION_TARGET_PAGE_LEFT:
@@ -1585,8 +1635,6 @@ static ClutterActor* _xfdashboard_windows_view_focusable_find_selection(Xfdashbo
 
 		default:
 			{
-				gchar					*valueName;
-
 				valueName=xfdashboard_get_enum_value_name(XFDASHBOARD_TYPE_SELECTION_TARGET, inDirection);
 				g_critical(_("Focusable object %s does not handle selection direction of type %s."),
 							G_OBJECT_TYPE_NAME(self),
@@ -1748,6 +1796,12 @@ static void _xfdashboard_windows_view_dispose(GObject *inObject)
 	XfdashboardWindowsViewPrivate	*priv=XFDASHBOARD_WINDOWS_VIEW(self)->priv;
 
 	/* Release allocated resources */
+	if(priv->selectedItem)
+	{
+		g_object_remove_weak_pointer(G_OBJECT(priv->selectedItem), &priv->selectedItem);
+		priv->selectedItem=NULL;
+	}
+
 	if(priv->scrollEventChangingWorkspaceStage)
 	{
 		if(priv->scrollEventChangingWorkspaceStageSignalID)
@@ -1837,6 +1891,10 @@ static void _xfdashboard_windows_view_set_property(GObject *inObject,
 			_xfdashboard_windows_view_set_filter_monitor_windows(self, g_value_get_boolean(inValue));
 			break;
 
+		case PROP_FILTER_WORKSPACE_WINDOWS:
+			_xfdashboard_windows_view_set_filter_workspace_windows(self, g_value_get_boolean(inValue));
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(inObject, inPropID, inSpec);
 			break;
@@ -1873,6 +1931,10 @@ static void _xfdashboard_windows_view_get_property(GObject *inObject,
 			g_value_set_boolean(outValue, self->priv->filterMonitorWindows);
 			break;
 
+		case PROP_FILTER_WORKSPACE_WINDOWS:
+			g_value_set_boolean(outValue, self->priv->filterWorkspaceWindows);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(inObject, inPropID, inSpec);
 			break;
@@ -1951,12 +2013,20 @@ static void xfdashboard_windows_view_class_init(XfdashboardWindowsViewClass *kla
 								FALSE,
 								G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+	XfdashboardWindowsViewProperties[PROP_FILTER_WORKSPACE_WINDOWS]=
+		g_param_spec_boolean("filter-workspace-windows",
+								_("Filter workspace windows"),
+								_("Whether this view should only show windows of active workspace"),
+								TRUE,
+								G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
 	g_object_class_install_properties(gobjectClass, PROP_LAST, XfdashboardWindowsViewProperties);
 
 	/* Define stylable properties */
 	xfdashboard_actor_install_stylable_property(actorClass, XfdashboardWindowsViewProperties[PROP_SPACING]);
 	xfdashboard_actor_install_stylable_property(actorClass, XfdashboardWindowsViewProperties[PROP_PREVENT_UPSCALING]);
 	xfdashboard_actor_install_stylable_property(actorClass, XfdashboardWindowsViewProperties[PROP_FILTER_MONITOR_WINDOWS]);
+	xfdashboard_actor_install_stylable_property(actorClass, XfdashboardWindowsViewProperties[PROP_FILTER_WORKSPACE_WINDOWS]);
 
 	/* Define actions */
 	XfdashboardWindowsViewSignals[ACTION_WINDOW_CLOSE]=
@@ -2165,6 +2235,7 @@ static void xfdashboard_windows_view_init(XfdashboardWindowsView *self)
 	priv->scrollEventChangingWorkspaceStage=NULL;
 	priv->scrollEventChangingWorkspaceStageSignalID=0;
 	priv->filterMonitorWindows=FALSE;
+	priv->filterWorkspaceWindows=TRUE;
 	priv->currentStage=NULL;
 	priv->currentMonitor=NULL;
 	priv->currentStageMonitorBindingID=0;
